// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KPLOTTING_VERSION_H
#define KPLOTTING_VERSION_H

#define KPLOTTING_VERSION_STRING "6.23.0"
#define KPLOTTING_VERSION_MAJOR 6
#define KPLOTTING_VERSION_MINOR 23
#define KPLOTTING_VERSION_PATCH 0
#define KPLOTTING_VERSION ((6<<16)|(23<<8)|(0))

#endif
